@@ -1,11 +1,11 @@
-// $Id: NodePanel.java,v 1.17 2009/10/30 03:00:51 cmzmasek Exp $
+// $Id:
 // FORESTER -- software libraries and applications
 // for evolutionary biology research and applications.
 //
 // Copyright (C) 2008-2009 Christian M. Zmasek
 // Copyright (C) 2008-2009 Burnham Institute for Medical Research
 // All rights reserved
-// 
+//
 // This library is free software; you can redistribute it and/or
 // modify it under the terms of the GNU Lesser General Public
 // License as published by the Free Software Foundation; either
@@ -15,18 +15,20 @@
 // but WITHOUT ANY WARRANTY; without even the implied warranty of
 // MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU
 // Lesser General Public License for more details.
-// 
+//
 // You should have received a copy of the GNU Lesser General Public
 // License along with this library; if not, write to the Free Software
 // Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301, USA
 //
-// Contact: cmzmasek@yahoo.com
+// Contact: phylosoft @ gmail . com
 // WWW: www.phylosoft.org/
 
 package org.forester.archaeopteryx;
 
+import java.awt.Color;
 import java.util.List;
 import java.util.SortedMap;
+import java.util.SortedSet;
 
 import javax.swing.JEditorPane;
 import javax.swing.JPanel;
@@ -43,20 +45,25 @@ import org.forester.phylogeny.PhylogenyMethods;
 import org.forester.phylogeny.PhylogenyNode;
 import org.forester.phylogeny.data.Annotation;
 import org.forester.phylogeny.data.BinaryCharacters;
+import org.forester.phylogeny.data.BranchWidth;
 import org.forester.phylogeny.data.Date;
 import org.forester.phylogeny.data.Distribution;
 import org.forester.phylogeny.data.Event;
 import org.forester.phylogeny.data.PhylogenyData;
+import org.forester.phylogeny.data.PhylogenyDataUtil;
+import org.forester.phylogeny.data.Point;
 import org.forester.phylogeny.data.PropertiesMap;
 import org.forester.phylogeny.data.Property;
 import org.forester.phylogeny.data.Reference;
 import org.forester.phylogeny.data.Sequence;
 import org.forester.phylogeny.data.Taxonomy;
+import org.forester.phylogeny.data.Uri;
 import org.forester.util.ForesterUtil;
 
 class NodePanel extends JPanel implements TreeSelectionListener {
 
     static final String       DIST_ALTITUDE            = "Altitude";
+    static final String       DIST_ALT_UNIT            = "Altitude unit";
     static final String       DIST_LONGITUDE           = "Longitude";
     static final String       DIST_LATITUDE            = "Latitude";
     static final String       DIST_GEODETIC_DATUM      = "Geodetic datum";
@@ -88,6 +95,8 @@ class NodePanel extends JPanel implements TreeSelectionListener {
     static final String       SEQ_SYMBOL               = "Symbol";
     static final String       SEQ_URI                  = "URI";
     static final String       NODE_BRANCH_LENGTH       = "Branch length";
+    static final String       NODE_BRANCH_WIDTH        = "Branch width";
+    static final String       NODE_BRANCH_COLOR        = "Branch color";
     static final String       NODE_NAME                = "Name";
     static final String       TAXONOMY_URI             = "URI";
     static final String       TAXONOMY_RANK            = "Rank";
@@ -106,8 +115,8 @@ class NodePanel extends JPanel implements TreeSelectionListener {
 
     public NodePanel( final PhylogenyNode phylogeny_node ) {
         String node_name = "";
-        if ( !ForesterUtil.isEmpty( phylogeny_node.getNodeName() ) ) {
-            node_name = phylogeny_node.getNodeName() + " ";
+        if ( !ForesterUtil.isEmpty( phylogeny_node.getName() ) ) {
+            node_name = phylogeny_node.getName() + " ";
         }
         final DefaultMutableTreeNode top = new DefaultMutableTreeNode( "Node " + node_name );
         createNodes( top, phylogeny_node );
@@ -165,8 +174,17 @@ class NodePanel extends JPanel implements TreeSelectionListener {
         }
     }
 
+    private static void addUri( final DefaultMutableTreeNode top, final Uri uri, final String name ) {
+        DefaultMutableTreeNode category;
+        category = new DefaultMutableTreeNode( name );
+        top.add( category );
+        addSubelement( category, "Description", uri.getDescription() );
+        addSubelement( category, "Type", uri.getType() );
+        addSubelement( category, "URI", uri.getValue().toString() );
+    }
+
     private static void addAnnotations( final DefaultMutableTreeNode top,
-                                        final List<PhylogenyData> annotations,
+                                        final SortedSet<Annotation> annotations,
                                         final DefaultMutableTreeNode category ) {
         if ( ( annotations != null ) && ( annotations.size() > 0 ) ) {
             category.add( new DefaultMutableTreeNode( "Annotations" ) );
@@ -178,15 +196,52 @@ class NodePanel extends JPanel implements TreeSelectionListener {
         }
     }
 
+    private static void addUris( final DefaultMutableTreeNode top,
+                                 final List<Uri> uris,
+                                 final DefaultMutableTreeNode category ) {
+        if ( ( uris != null ) && ( uris.size() > 0 ) ) {
+            category.add( new DefaultMutableTreeNode( "URIs" ) );
+            final DefaultMutableTreeNode last = top.getLastLeaf();
+            int i = 0;
+            for( final Uri uri : uris ) {
+                if ( uri != null ) {
+                    addUri( last, uri, "URI " + ( i++ ) );
+                }
+            }
+        }
+    }
+
+    private static void addLineage( final DefaultMutableTreeNode top,
+                                    final List<String> lineage,
+                                    final DefaultMutableTreeNode category ) {
+        if ( ( lineage != null ) && ( lineage.size() > 0 ) ) {
+            final StringBuilder sb = new StringBuilder();
+            for( final String lin : lineage ) {
+                if ( !ForesterUtil.isEmpty( lin ) ) {
+                    sb.append( lin );
+                    sb.append( " > " );
+                }
+            }
+            String str = null;
+            if ( sb.length() > 1 ) {
+                str = sb.substring( 0, sb.length() - 3 );
+            }
+            if ( !ForesterUtil.isEmpty( str ) ) {
+                addSubelement( category, "Lineage", str );
+            }
+        }
+    }
+
     private static void addBasics( final DefaultMutableTreeNode top,
                                    final PhylogenyNode phylogeny_node,
                                    final String name ) {
         final DefaultMutableTreeNode category = new DefaultMutableTreeNode( name );
         top.add( category );
-        addSubelement( category, NODE_NAME, phylogeny_node.getNodeName() );
-        if ( phylogeny_node.getDistanceToParent() != PhylogenyNode.DISTANCE_DEFAULT ) {
-            addSubelement( category, NODE_BRANCH_LENGTH, ForesterUtil.FORMATTER_6.format( phylogeny_node
-                    .getDistanceToParent() ) );
+        addSubelement( category, NODE_NAME, phylogeny_node.getName() );
+        if ( phylogeny_node.getDistanceToParent() != PhylogenyDataUtil.BRANCH_LENGTH_DEFAULT ) {
+            addSubelement( category,
+                           NODE_BRANCH_LENGTH,
+                           ForesterUtil.FORMATTER_6.format( phylogeny_node.getDistanceToParent() ) );
         }
         if ( phylogeny_node.getBranchData().isHasConfidences() ) {
             for( final PhylogenyData conf : phylogeny_node.getBranchData().getConfidences() ) {
@@ -195,11 +250,18 @@ class NodePanel extends JPanel implements TreeSelectionListener {
         }
         if ( !phylogeny_node.isExternal() ) {
             addSubelement( category, "Children", String.valueOf( phylogeny_node.getNumberOfDescendants() ) );
-            addSubelement( category, "External children", String.valueOf( phylogeny_node.getAllExternalDescendants()
-                    .size() ) );
-            final int tax_count = PhylogenyMethods.calculateSumOfDistinctTaxonomies( phylogeny_node );
-            if ( tax_count > 0 ) {
+            addSubelement( category,
+                           "External children",
+                           String.valueOf( phylogeny_node.getAllExternalDescendants().size() ) );
+            final SortedMap<Taxonomy, Integer> distinct_tax = PhylogenyMethods
+                    .obtainDistinctTaxonomyCounts( phylogeny_node );
+            if ( distinct_tax != null ) {
+                final int no_tax = PhylogenyMethods.calculateNumberOfExternalNodesWithoutTaxonomy( phylogeny_node );
+                final int tax_count = distinct_tax.size();
                 addSubelement( category, "Distinct external taxonomies", String.valueOf( tax_count ) );
+                if ( no_tax > 0 ) {
+                    addSubelement( category, "External nodes without taxonomy", String.valueOf( no_tax ) );
+                }
             }
         }
         if ( !phylogeny_node.isRoot() ) {
@@ -209,6 +271,16 @@ class NodePanel extends JPanel implements TreeSelectionListener {
                 addSubelement( category, "Distance to root", String.valueOf( ForesterUtil.FORMATTER_6.format( d ) ) );
             }
         }
+        if ( ( phylogeny_node.getBranchData().getBranchWidth() != null )
+                && ( phylogeny_node.getBranchData().getBranchWidth().getValue() != BranchWidth.BRANCH_WIDTH_DEFAULT_VALUE ) ) {
+            addSubelement( category,
+                           NODE_BRANCH_WIDTH,
+                           ForesterUtil.FORMATTER_3.format( phylogeny_node.getBranchData().getBranchWidth().getValue() ) );
+        }
+        if ( ( phylogeny_node.getBranchData().getBranchColor() != null ) ) {
+            final Color c = phylogeny_node.getBranchData().getBranchColor().getValue();
+            addSubelement( category, NODE_BRANCH_COLOR, c.getRed() + ", " + c.getGreen() + ", " + c.getBlue() );
+        }
     }
 
     private static void addBinaryCharacters( final DefaultMutableTreeNode top,
@@ -243,10 +315,19 @@ class NodePanel extends JPanel implements TreeSelectionListener {
         category = new DefaultMutableTreeNode( name );
         top.add( category );
         addSubelement( category, DIST_DESCRIPTION, dist.getDesc() );
-        addSubelement( category, DIST_GEODETIC_DATUM, dist.getGeodeticDatum() );
-        addSubelement( category, DIST_LATITUDE, String.valueOf( dist.getLatitude() ) );
-        addSubelement( category, DIST_LONGITUDE, String.valueOf( dist.getLongitude() ) );
-        addSubelement( category, DIST_ALTITUDE, String.valueOf( dist.getAltitude() ) );
+        if ( ( dist.getPoints() != null ) && ( dist.getPoints().size() > 0 ) ) {
+            final Point p0 = dist.getPoints().get( 0 );
+            if ( p0 != null ) {
+                addSubelement( category, DIST_GEODETIC_DATUM, p0.getGeodeticDatum() );
+                addSubelement( category, DIST_LATITUDE, String.valueOf( p0.getLatitude() ) );
+                addSubelement( category, DIST_LONGITUDE, String.valueOf( p0.getLongitude() ) );
+                String alt_unit = p0.getAltiudeUnit();
+                if ( ForesterUtil.isEmpty( alt_unit ) ) {
+                    alt_unit = "?";
+                }
+                addSubelement( category, DIST_ALTITUDE, String.valueOf( p0.getAltitude() ) + alt_unit );
+            }
+        }
     }
 
     private static void addEvents( final DefaultMutableTreeNode top, final Event events, final String name ) {
@@ -276,7 +357,7 @@ class NodePanel extends JPanel implements TreeSelectionListener {
         top.add( category );
         for( final String key : properties_map.keySet() ) {
             final Property prop = properties_map.get( key );
-            category.add( new DefaultMutableTreeNode( prop.getRef() + " " + prop.getValue() + " " + prop.getUnit()
+            category.add( new DefaultMutableTreeNode( prop.getRef() + "=" + prop.getValue() + " " + prop.getUnit()
                     + " [" + prop.getAppliesTo().toString() + "]" ) );
         }
     }
@@ -285,7 +366,7 @@ class NodePanel extends JPanel implements TreeSelectionListener {
         final DefaultMutableTreeNode category = new DefaultMutableTreeNode( name );
         top.add( category );
         addSubelement( category, LIT_REFERENCE_DOI, ref.getDoi() );
-        addSubelement( category, LIT_REFERENCE_DESC, ref.getValue() );
+        addSubelement( category, LIT_REFERENCE_DESC, ref.getDescription() );
     }
 
     private static void addSequence( final DefaultMutableTreeNode top, final Sequence seq, final String name ) {
@@ -299,8 +380,8 @@ class NodePanel extends JPanel implements TreeSelectionListener {
         addSubelement( category, SEQ_LOCATION, seq.getLocation() );
         addSubelement( category, SEQ_TYPE, seq.getType() );
         addSubelement( category, SEQ_MOL_SEQ, seq.getMolecularSequence() );
-        if ( seq.getUri() != null ) {
-            addSubelement( category, NodePanel.SEQ_URI, seq.getUri().toString() );
+        if ( ( seq.getUris() != null ) && !seq.getUris().isEmpty() ) {
+            addUris( top, seq.getUris(), category );
         }
         addAnnotations( top, seq.getAnnotations(), category );
     }
@@ -325,8 +406,11 @@ class NodePanel extends JPanel implements TreeSelectionListener {
             addSubelement( category, TAXONOMY_SYNONYM, syn );
         }
         addSubelement( category, TAXONOMY_RANK, tax.getRank() );
-        if ( tax.getUri() != null ) {
-            addSubelement( category, TAXONOMY_URI, tax.getUri().toString() );
+        if ( ( tax.getUris() != null ) && !tax.getUris().isEmpty() ) {
+            addUris( top, tax.getUris(), category );
+        }
+        if ( ( tax.getLineage() != null ) && !tax.getLineage().isEmpty() ) {
+            addLineage( top, tax.getLineage(), category );
         }
     }
 
