@@ -1,11 +1,11 @@
-// $Id: Options.java,v 1.28 2009/12/09 20:13:50 cmzmasek Exp $
+// $Id:
 // FORESTER -- software libraries and applications
 // for evolutionary biology research and applications.
 //
 // Copyright (C) 2009 Christian M. Zmasek
 // Copyright (C) 2009 Burnham Institute for Medical Research
 // All rights reserved
-// 
+//
 // This library is free software; you can redistribute it and/or
 // modify it under the terms of the GNU Lesser General Public
 // License as published by the Free Software Foundation; either
@@ -15,57 +15,69 @@
 // but WITHOUT ANY WARRANTY; without even the implied warranty of
 // MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU
 // Lesser General Public License for more details.
-// 
+//
 // You should have received a copy of the GNU Lesser General Public
 // License along with this library; if not, write to the Free Software
 // Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301, USA
 //
-// Contact: cmzmasek@yahoo.com
+// Contact: phylosoft @ gmail . com
 // WWW: www.phylosoft.org/forester
 
 package org.forester.archaeopteryx;
 
 import java.awt.Font;
 
+import org.forester.phylogeny.PhylogenyNodeI.NH_CONVERSION_SUPPORT_VALUE_STYLE;
+import org.forester.phylogeny.data.NodeVisualization;
+import org.forester.phylogeny.data.NodeVisualization.NodeFill;
+import org.forester.phylogeny.data.NodeVisualization.NodeShape;
 import org.forester.util.ForesterUtil;
 
 /*
  * This is to hold changeable options.
  */
-final class Options {
+final public class Options {
 
-    static final double             MIN_CONFIDENCE_DEFAULT = 0.0;
-    private boolean                 _show_node_boxes;
-    private boolean                 _show_branch_length_values;
-    private boolean                 _internal_number_are_confidence_for_nh_parsing;
-    private boolean                 _show_scale;
-    private boolean                 _show_overview;
-    private boolean                 _antialias_screen;
-    private boolean                 _antialias_print;
-    private boolean                 _graphics_export_visible_only;
-    private int                     _print_size_x;
-    private int                     _print_size_y;
-    private double                  _min_confidence_value;
-    private boolean                 _print_black_and_white;
-    private boolean                 _print_using_actual_size;
-    private boolean                 _graphics_export_using_actual_size;
-    private PHYLOGENY_GRAPHICS_TYPE _phylogeny_graphics_type;
-    private CLADOGRAM_TYPE          _cladogram_type;
-    private OVERVIEW_PLACEMENT_TYPE _ov_placement;
-    private NODE_LABEL_DIRECTION    _node_label_direction;
-    private Font                    _base_font;
-    private boolean                 _match_whole_terms_only;
-    private boolean                 _search_case_sensitive;
-    private float                   _print_line_width;
-    private boolean                 _inverse_search_result;
-    private double                  _scale_bar_length;
-    private short                   _number_of_digits_after_comma_for_confidence_values;
-    private short                   _number_of_digits_after_comma_for_branch_length_values;
-    private boolean                 _nh_parsing_replace_underscores;
-    private boolean                 _nh_parsing_extract_pfam_taxonomy_codes;
-    private boolean                 _nh_parsing_ignore_quotes;
-    private boolean                 _editable;
-    private boolean                 _background_color_gradient;
+    static final double                       MIN_CONFIDENCE_DEFAULT = 0.0;
+    private boolean                           _show_branch_length_values;
+    private boolean                           _internal_number_are_confidence_for_nh_parsing;
+    private boolean                           _show_scale;
+    private boolean                           _show_overview;
+    private boolean                           _antialias_screen;
+    private boolean                           _antialias_print;
+    private boolean                           _graphics_export_visible_only;
+    private int                               _print_size_x;
+    private int                               _print_size_y;
+    private double                            _min_confidence_value;
+    private boolean                           _print_black_and_white;
+    private boolean                           _print_using_actual_size;
+    private boolean                           _graphics_export_using_actual_size;
+    private PHYLOGENY_GRAPHICS_TYPE           _phylogeny_graphics_type;
+    private CLADOGRAM_TYPE                    _cladogram_type;
+    private OVERVIEW_PLACEMENT_TYPE           _ov_placement;
+    private NODE_LABEL_DIRECTION              _node_label_direction;
+    private Font                              _base_font;
+    private boolean                           _match_whole_terms_only;
+    private boolean                           _search_case_sensitive;
+    private float                             _print_line_width;
+    private boolean                           _inverse_search_result;
+    private double                            _scale_bar_length;
+    private short                             _number_of_digits_after_comma_for_confidence_values;
+    private short                             _number_of_digits_after_comma_for_branch_length_values;
+    private boolean                           _nh_parsing_replace_underscores;
+    private boolean                           _nh_parsing_extract_pfam_taxonomy_codes;
+    private boolean                           _editable;
+    private boolean                           _background_color_gradient;
+    private boolean                           _show_domain_labels;
+    private boolean                           _color_labels_same_as_parent_branch;
+    private boolean                           _abbreviate_scientific_names;
+    private NodeVisualization.NodeShape       _default_node_shape;
+    private NodeVisualization.NodeFill        _default_node_fill;
+    private short                             _default_node_shape_size;
+    private boolean                           _taxonomy_colorize_node_shapes;
+    private boolean                           _show_default_node_shapes;
+    private boolean                           _show_confidence_stddev;
+    private NH_CONVERSION_SUPPORT_VALUE_STYLE _nh_conversion_support_value_style;
 
     private Options() {
         init();
@@ -79,6 +91,18 @@ final class Options {
         return _cladogram_type;
     }
 
+    final NodeFill getDefaultNodeFill() {
+        return _default_node_fill;
+    }
+
+    final NodeShape getDefaultNodeShape() {
+        return _default_node_shape;
+    }
+
+    final short getDefaultNodeShapeSize() {
+        return _default_node_shape_size;
+    }
+
     final double getMinConfidenceValue() {
         return _min_confidence_value;
     }
@@ -120,7 +144,10 @@ final class Options {
     }
 
     final private void init() {
-        _show_node_boxes = false;
+        _default_node_shape = NodeShape.CIRCLE;
+        _default_node_fill = NodeFill.GRADIENT;
+        _default_node_shape_size = Constants.DEFAULT_NODE_SHAPE_SIZE_DEFAULT;
+        _taxonomy_colorize_node_shapes = false;
         _show_branch_length_values = false;
         _internal_number_are_confidence_for_nh_parsing = false;
         _show_scale = false;
@@ -129,7 +156,8 @@ final class Options {
         _graphics_export_visible_only = false;
         _editable = true;
         _background_color_gradient = false;
-        if ( Util.isUsOrCanada() ) {
+        _show_default_node_shapes = false;
+        if ( AptxUtil.isUsOrCanada() ) {
             _print_size_x = Constants.US_LETTER_SIZE_X;
             _print_size_y = Constants.US_LETTER_SIZE_Y;
         }
@@ -155,8 +183,36 @@ final class Options {
         _number_of_digits_after_comma_for_confidence_values = Constants.NUMBER_OF_DIGITS_AFTER_COMMA_FOR_CONFIDENCE_VALUES_DEFAULT;
         _nh_parsing_replace_underscores = false;
         _nh_parsing_extract_pfam_taxonomy_codes = false;
-        _nh_parsing_ignore_quotes = Constants.NH_PARSING_IGNORE_QUOTES_DEFAULT;
         _cladogram_type = Constants.CLADOGRAM_TYPE_DEFAULT;
+        _show_domain_labels = true;
+        setAbbreviateScientificTaxonNames( false );
+        _color_labels_same_as_parent_branch = false;
+        _show_confidence_stddev = true;
+        _nh_conversion_support_value_style = NH_CONVERSION_SUPPORT_VALUE_STYLE.NONE;
+    }
+
+    boolean isShowConfidenceStddev() {
+        return _show_confidence_stddev;
+    }
+
+    void setShowConfidenceStddev( final boolean show_confidence_stddev ) {
+        _show_confidence_stddev = show_confidence_stddev;
+    }
+
+    NH_CONVERSION_SUPPORT_VALUE_STYLE getNhConversionSupportValueStyle() {
+        return _nh_conversion_support_value_style;
+    }
+
+    void setNhConversionSupportValueStyle( final NH_CONVERSION_SUPPORT_VALUE_STYLE nh_conversion_support_value_style ) {
+        _nh_conversion_support_value_style = nh_conversion_support_value_style;
+    }
+
+    final boolean isAbbreviateScientificTaxonNames() {
+        return _abbreviate_scientific_names;
+    }
+
+    boolean isAllowMagnificationOfTaxonomyImages() {
+        return true;
     }
 
     final boolean isAntialiasPrint() {
@@ -167,6 +223,14 @@ final class Options {
         return _antialias_screen;
     }
 
+    final boolean isBackgroundColorGradient() {
+        return _background_color_gradient;
+    }
+
+    final boolean isColorLabelsSameAsParentBranch() {
+        return _color_labels_same_as_parent_branch;
+    }
+
     final boolean isEditable() {
         return _editable;
     }
@@ -195,10 +259,6 @@ final class Options {
         return _match_whole_terms_only;
     }
 
-    final boolean isNhParsingIgnoreQuotes() {
-        return _nh_parsing_ignore_quotes;
-    }
-
     final boolean isPrintBlackAndWhite() {
         return _print_black_and_white;
     }
@@ -219,8 +279,8 @@ final class Options {
         return _show_branch_length_values;
     }
 
-    final boolean isShowNodeBoxes() {
-        return _show_node_boxes;
+    public final boolean isShowDomainLabels() {
+        return _show_domain_labels;
     }
 
     final boolean isShowOverview() {
@@ -231,6 +291,14 @@ final class Options {
         return _show_scale;
     }
 
+    boolean isTaxonomyColorizeNodeShapes() {
+        return _taxonomy_colorize_node_shapes;
+    }
+
+    final void setAbbreviateScientificTaxonNames( final boolean abbreviate_scientific_names ) {
+        _abbreviate_scientific_names = abbreviate_scientific_names;
+    }
+
     final void setAntialiasPrint( final boolean antialias_print ) {
         _antialias_print = antialias_print;
     }
@@ -239,6 +307,10 @@ final class Options {
         _antialias_screen = antialias_screen;
     }
 
+    public void setBackgroundColorGradient( final boolean background_color_gradient ) {
+        _background_color_gradient = background_color_gradient;
+    }
+
     final void setBaseFont( final Font base_font ) {
         _base_font = base_font;
     }
@@ -247,6 +319,22 @@ final class Options {
         _cladogram_type = cladogram_type;
     }
 
+    public void setColorLabelsSameAsParentBranch( final boolean color_labels_same_as_parent_branch ) {
+        _color_labels_same_as_parent_branch = color_labels_same_as_parent_branch;
+    }
+
+    final void setDefaultNodeFill( final NodeFill default_node_fill ) {
+        _default_node_fill = default_node_fill;
+    }
+
+    final void setDefaultNodeShape( final NodeShape default_node_shape ) {
+        _default_node_shape = default_node_shape;
+    }
+
+    final void setDefaultNodeShapeSize( final short default_node_shape_size ) {
+        _default_node_shape_size = default_node_shape_size;
+    }
+
     final void setEditable( final boolean editable ) {
         _editable = editable;
     }
@@ -285,10 +373,6 @@ final class Options {
         _min_confidence_value = min_confidence_value;
     }
 
-    final void setNhParsingIgnoreQuotes( final boolean nh_parsing_ignore_quotes ) {
-        _nh_parsing_ignore_quotes = nh_parsing_ignore_quotes;
-    }
-
     final void setNodeLabelDirection( final NODE_LABEL_DIRECTION node_label_direction ) {
         _node_label_direction = node_label_direction;
     }
@@ -345,8 +429,8 @@ final class Options {
         _show_branch_length_values = show_branch_length_values;
     }
 
-    final void setShowNodeBoxes( final boolean show_node_boxes ) {
-        _show_node_boxes = show_node_boxes;
+    public void setShowDomainLabels( final boolean show_domain_labels ) {
+        _show_domain_labels = show_domain_labels;
     }
 
     final void setShowOverview( final boolean show_overview ) {
@@ -357,11 +441,23 @@ final class Options {
         _show_scale = show_scale;
     }
 
+    void setTaxonomyColorizeNodeShapes( final boolean taxonomy_colorize_node_shapes ) {
+        _taxonomy_colorize_node_shapes = taxonomy_colorize_node_shapes;
+    }
+
     final static Options createDefaultInstance() {
         return new Options();
     }
 
-    final static Options createInstance( final Configuration configuration ) {
+    boolean isShowDefaultNodeShapes() {
+        return _show_default_node_shapes;
+    }
+
+    void setShowDefaultNodeShapes( final boolean show_default_node_shapes ) {
+        _show_default_node_shapes = show_default_node_shapes;
+    }
+
+    public final static Options createInstance( final Configuration configuration ) {
         final Options instance = createDefaultInstance();
         if ( configuration != null ) {
             instance.setAntialiasScreen( configuration.isAntialiasScreen() );
@@ -385,8 +481,10 @@ final class Options {
             instance.setReplaceUnderscoresInNhParsing( configuration.isReplaceUnderscoresInNhParsing() );
             instance.setInternalNumberAreConfidenceForNhParsing( configuration
                     .isInternalNumberAreConfidenceForNhParsing() );
-            instance.setNhParsingIgnoreQuotes( configuration.isNhParsingIgnoreQuotes() );
             instance.setEditable( configuration.isEditable() );
+            instance.setColorLabelsSameAsParentBranch( configuration.isColorLabelsSameAsParentBranch() );
+            instance.setShowDomainLabels( configuration.isShowDomainLabels() );
+            instance.setAbbreviateScientificTaxonNames( configuration.isAbbreviateScientificTaxonNames() );
             if ( configuration.getMinConfidenceValue() != MIN_CONFIDENCE_DEFAULT ) {
                 instance.setMinConfidenceValue( configuration.getMinConfidenceValue() );
             }
@@ -406,15 +504,26 @@ final class Options {
             if ( configuration.getPhylogenyGraphicsType() != null ) {
                 instance.setPhylogenyGraphicsType( configuration.getPhylogenyGraphicsType() );
             }
+            if ( configuration.getDefaultNodeFill() != null ) {
+                instance.setDefaultNodeFill( configuration.getDefaultNodeFill() );
+            }
+            if ( configuration.getDefaultNodeShape() != null ) {
+                instance.setDefaultNodeShape( configuration.getDefaultNodeShape() );
+            }
+            if ( configuration.getDefaultNodeShapeSize() >= 0 ) {
+                instance.setDefaultNodeShapeSize( configuration.getDefaultNodeShapeSize() );
+            }
+            instance.setTaxonomyColorizeNodeShapes( configuration.isTaxonomyColorizeNodeShapes() );
+            instance.setShowDefaultNodeShapes( configuration.isShowDefaultNodeShapes() );
         }
         return instance;
     }
 
-    static enum CLADOGRAM_TYPE {
+    public static enum CLADOGRAM_TYPE {
         NON_LINED_UP, EXT_NODE_SUM_DEP, TOTAL_NODE_SUM_DEP;
     }
 
-    static enum NODE_LABEL_DIRECTION {
+    public static enum NODE_LABEL_DIRECTION {
         HORIZONTAL, RADIAL;
     }
 
@@ -440,15 +549,7 @@ final class Options {
         }
     }
 
-    static enum PHYLOGENY_GRAPHICS_TYPE {
+    public static enum PHYLOGENY_GRAPHICS_TYPE {
         RECTANGULAR, TRIANGULAR, EURO_STYLE, ROUNDED, CONVEX, CURVED, UNROOTED, CIRCULAR;
     }
-
-    public void setBackgroundColorGradient( final boolean background_color_gradient ) {
-        _background_color_gradient = background_color_gradient;
-    }
-
-    public boolean isBackgroundColorGradient() {
-        return _background_color_gradient;
-    }
 }
