@@ -1,4 +1,4 @@
-// $Id: PhyloXmlDataFormatException.java,v 1.1 2009/10/26 22:38:19 cmzmasek Exp
+// $Id:
 // $
 // FORESTER -- software libraries and applications
 // for evolutionary biology research and applications.
@@ -6,7 +6,7 @@
 // Copyright (C) 2008-2009 Christian M. Zmasek
 // Copyright (C) 2008-2009 Burnham Institute for Medical Research
 // All rights reserved
-// 
+//
 // This library is free software; you can redistribute it and/or
 // modify it under the terms of the GNU Lesser General Public
 // License as published by the Free Software Foundation; either
@@ -16,17 +16,19 @@
 // but WITHOUT ANY WARRANTY; without even the implied warranty of
 // MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU
 // Lesser General Public License for more details.
-// 
+//
 // You should have received a copy of the GNU Lesser General Public
 // License along with this library; if not, write to the Free Software
 // Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301, USA
 //
-// Contact: cmzmasek@yahoo.com
+// Contact: phylosoft @ gmail . com
 // WWW: www.phylosoft.org/forester
 
 package org.forester.io.parsers.phyloxml;
 
-public class PhyloXmlDataFormatException extends PhyloXmlException {
+import java.io.IOException;
+
+public class PhyloXmlDataFormatException extends IOException {
 
     private static final long serialVersionUID = 3756209394438250170L;
 
