@@ -1,4 +1,4 @@
-// $Id: PhylogenyNode.java,v 1.127 2009/11/17 03:51:34 cmzmasek Exp $
+// $Id:
 // FORESTER -- software libraries and applications
 // for evolutionary biology research and applications.
 //
@@ -7,7 +7,7 @@
 // Copyright (C) 2000-2001 Washington University School of Medicine
 // and Howard Hughes Medical Institute
 // All rights reserved
-// 
+//
 // This library is free software; you can redistribute it and/or
 // modify it under the terms of the GNU Lesser General Public
 // License as published by the Free Software Foundation; either
@@ -17,12 +17,12 @@
 // but WITHOUT ANY WARRANTY; without even the implied warranty of
 // MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU
 // Lesser General Public License for more details.
-// 
+//
 // You should have received a copy of the GNU Lesser General Public
 // License along with this library; if not, write to the Free Software
 // Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301, USA
 //
-// Contact: cmzmasek@yahoo.com
+// Contact: phylosoft @ gmail . com
 // WWW: www.phylosoft.org/forester
 
 package org.forester.phylogeny;
@@ -30,29 +30,32 @@ package org.forester.phylogeny;
 import java.util.ArrayList;
 import java.util.List;
 
-import org.forester.io.parsers.PhylogenyParserException;
 import org.forester.io.parsers.nhx.NHXFormatException;
 import org.forester.io.parsers.nhx.NHXParser;
-import org.forester.io.parsers.nhx.NHXParser.TAXONOMY_EXTRACTION;
+import org.forester.io.parsers.phyloxml.PhyloXmlDataFormatException;
+import org.forester.io.parsers.phyloxml.PhyloXmlUtil;
 import org.forester.phylogeny.data.BranchData;
+import org.forester.phylogeny.data.Confidence;
 import org.forester.phylogeny.data.NodeData;
-import org.forester.phylogeny.iterators.ChildNodeIteratorForward;
-import org.forester.phylogeny.iterators.PhylogenyNodeIterator;
+import org.forester.phylogeny.data.PhylogenyDataUtil;
 import org.forester.phylogeny.iterators.PreorderTreeIterator;
 import org.forester.util.ForesterUtil;
 
-public class PhylogenyNode implements PhylogenyNodeI, Comparable<PhylogenyNode> {
+/**
+ * Warning. Implementation of method 'compareTo' only looks at 
+ * node name. Thus, use of this class in SortedSets might lead
+ * to unexpected behavior.
+ *
+ */
+public final class PhylogenyNode implements PhylogenyNodeI, Comparable<PhylogenyNode> {
 
-    /** Value of -99.0 is used as default value. */
-    public final static double       DISTANCE_DEFAULT = -1024.0;
     private static int               _node_count      = 0;
     private byte                     _indicator;
     private int                      _id;
     private int                      _sum_ext_nodes;
     private float                    _x;
     private float                    _y;
-    private String                   _node_name;
-    private double                   _distance_parent;
+    private double                   _distance_parent = PhylogenyDataUtil.BRANCH_LENGTH_DEFAULT;
     private boolean                  _collapse;
     private PhylogenyNode            _parent;
     private PhylogenyNode            _link;
@@ -62,56 +65,20 @@ public class PhylogenyNode implements PhylogenyNodeI, Comparable<PhylogenyNode>
     private float                    _x_secondary;
     private float                    _y_secondary;
 
-    private boolean                  _pathToParent = false;
-
     /**
      * Default constructor for PhylogenyNode.
      */
     public PhylogenyNode() {
-        init();
-        setNodeId( PhylogenyNode.getNodeCount() );
+        //  init();
+        setId( PhylogenyNode.getNodeCount() );
         PhylogenyNode.increaseNodeCount();
         setSumExtNodes( 1 ); // For ext node, this number is 1 (not 0!!)
     }
 
-    public PhylogenyNode( final String nhx ) throws NHXFormatException {
-        this( nhx, TAXONOMY_EXTRACTION.NO );
-    }
-
-    public PhylogenyNode( final String nhx, final TAXONOMY_EXTRACTION taxonomy_extraction ) throws NHXFormatException {
-        init();
-        NHXParser.parseNHX( nhx, this, taxonomy_extraction, false );
-        setNodeId( PhylogenyNode.getNodeCount() );
-        PhylogenyNode.increaseNodeCount();
-        setSumExtNodes( 1 ); // For ext node, this number is 1 (not 0!!)
-    }
-
-    /**
-     * Constructor for PhylogenyNode.
-     * <p>
-     * 
-     * @param s
-     *            String representing one PhylogenyNode in New Hampshire (NH) or
-     *            New Hampshire X (NHX) format.
-     * @throws NHXFormatException
-     * @throws PhylogenyParserException
-     */
-    public PhylogenyNode( final String nhx,
-                          final TAXONOMY_EXTRACTION taxonomy_extraction,
-                          final boolean replace_underscores ) throws NHXFormatException {
-        init();
-        NHXParser.parseNHX( nhx, this, taxonomy_extraction, replace_underscores );
-        setNodeId( PhylogenyNode.getNodeCount() );
-        PhylogenyNode.increaseNodeCount();
-        setSumExtNodes( 1 ); // For ext node, this number is 1 (not 0!!)
-    }
-
-    public void setPathToParent(boolean value){
-        _pathToParent = value;
-    }
-
-    public boolean getPathToParent(){
-        return _pathToParent;
+    public void removeConnections() {
+        _parent = null;
+        _link = null;
+        _descendants = null;
     }
 
     /**
@@ -121,10 +88,9 @@ public class PhylogenyNode implements PhylogenyNodeI, Comparable<PhylogenyNode>
      * @param n
      *            the PhylogenyNode to add
      */
+    @Override
     final public void addAsChild( final PhylogenyNodeI node ) {
         final PhylogenyNode n = ( PhylogenyNode ) node;
-        // of
-        // interface!!!!
         addChildNode( n );
         n.setParent( this );
     }
@@ -141,12 +107,14 @@ public class PhylogenyNode implements PhylogenyNodeI, Comparable<PhylogenyNode>
         getDescendants().add( child );
     }
 
+    @Override
+    // this is poor, as it only compares on names!
     final public int compareTo( final PhylogenyNode o ) {
         final PhylogenyNode n = o;
-        if ( ( getNodeName() == null ) || ( n.getNodeName() == null ) ) {
+        if ( ( getName() == null ) || ( n.getName() == null ) ) {
             return 0;
         }
-        return getNodeName().compareTo( n.getNodeName() );
+        return getName().compareTo( n.getName() );
     }
 
     // ---------------------------------------------------------
@@ -167,9 +135,6 @@ public class PhylogenyNode implements PhylogenyNodeI, Comparable<PhylogenyNode>
         node._indicator = _indicator;
         node._x = _x;
         node._y = _y;
-        if ( _node_name != null ) {
-            node._node_name = new String( _node_name );
-        }
         node._distance_parent = _distance_parent;
         node._collapse = _collapse;
         node._link = _link;
@@ -180,7 +145,30 @@ public class PhylogenyNode implements PhylogenyNodeI, Comparable<PhylogenyNode>
             node._branch_data = ( BranchData ) _branch_data.copy();
         }
         return node;
-    } // copyNodeData()
+    }
+
+    /**
+     * Returns a new PhylogenyNode which has the same data as this
+     * PhylogenyNode. Links to the other Nodes in the same Phylogeny are NOT
+     * copied (e.g. _link to _parent). Field "_link" IS copied.
+     * 
+     * @see #getLink() 
+     */
+    final public PhylogenyNode copyNodeDataShallow() {
+        final PhylogenyNode node = new PhylogenyNode();
+        PhylogenyNode.decreaseNodeCount();
+        node._id = _id;
+        node._sum_ext_nodes = _sum_ext_nodes;
+        node._indicator = _indicator;
+        node._x = _x;
+        node._y = _y;
+        node._distance_parent = _distance_parent;
+        node._collapse = _collapse;
+        node._link = _link;
+        node._node_data = _node_data;
+        node._branch_data = _branch_data;
+        return node;
+    }
 
     @Override
     /**
@@ -201,7 +189,7 @@ public class PhylogenyNode implements PhylogenyNodeI, Comparable<PhylogenyNode>
         }
         else {
             final PhylogenyNode other = ( PhylogenyNode ) o;
-            if ( !getNodeName().equals( other.getNodeName() ) ) {
+            if ( !getName().equals( other.getName() ) ) {
                 return false;
             }
             final NodeData this_data = getNodeData();
@@ -211,13 +199,13 @@ public class PhylogenyNode implements PhylogenyNodeI, Comparable<PhylogenyNode>
                 return ( this_data.getTaxonomy().isEqual( other_data.getTaxonomy() ) && this_data.getSequence()
                         .isEqual( other_data.getSequence() ) );
             }
-            else if ( this_data.isHasSequence() && other_data.isHasSequence() ) {
-                return ( this_data.getSequence().isEqual( other_data.getSequence() ) );
-            }
             else if ( this_data.isHasTaxonomy() && other_data.isHasTaxonomy() ) {
                 return ( this_data.getTaxonomy().isEqual( other_data.getTaxonomy() ) );
             }
-            else if ( getNodeName().length() > 0 ) {
+            else if ( this_data.isHasSequence() && other_data.isHasSequence() ) {
+                return ( this_data.getSequence().isEqual( other_data.getSequence() ) );
+            }
+            else if ( getName().length() > 0 ) {
                 // Node name is not empty, and equal.
                 return true;
             }
@@ -250,13 +238,6 @@ public class PhylogenyNode implements PhylogenyNodeI, Comparable<PhylogenyNode>
         while ( !node2.isExternal() ) {
             node2 = node2.getLastChildNode();
         }
-        //  do {
-        //      nodes.add( node1 );
-        //      node1 = node1.getNextExternalNode();
-        //  } while ( ( node1 != node2 ) && ( node1 != null ) );
-        //  if ( node1 == null ) {
-        //      System.out.println( "node1 == null" );
-        //  }
         while ( node1 != node2 ) {
             nodes.add( node1 );
             node1 = node1.getNextExternalNode();
@@ -275,7 +256,7 @@ public class PhylogenyNode implements PhylogenyNodeI, Comparable<PhylogenyNode>
         final List<PhylogenyNode> c = getAllExternalDescendants();
         final List<String> n = new ArrayList<String>( c.size() );
         for( final PhylogenyNode phylogenyNode : c ) {
-            n.add( phylogenyNode.getNodeName() );
+            n.add( phylogenyNode.getName() );
         }
         return n;
     }
@@ -300,6 +281,7 @@ public class PhylogenyNode implements PhylogenyNodeI, Comparable<PhylogenyNode>
      * @throws IllegalArgumentException
      *             if n is out of bounds
      */
+    @Override
     final public PhylogenyNode getChildNode( final int i ) {
         if ( isExternal() ) {
             throw new UnsupportedOperationException( "attempt to get the child node of an external node." );
@@ -364,6 +346,9 @@ public class PhylogenyNode implements PhylogenyNodeI, Comparable<PhylogenyNode>
     }
 
     final public List<PhylogenyNode> getDescendants() {
+        if ( _descendants == null ) {
+            _descendants = new ArrayList<PhylogenyNode>();
+        }
         return _descendants;
     }
 
@@ -371,6 +356,7 @@ public class PhylogenyNode implements PhylogenyNodeI, Comparable<PhylogenyNode>
      * Returns the length of the branch leading to the _parent of this
      * PhylogenyNode (double).
      */
+    @Override
     final public double getDistanceToParent() {
         return _distance_parent;
     }
@@ -441,6 +427,49 @@ public class PhylogenyNode implements PhylogenyNodeI, Comparable<PhylogenyNode>
         return current_node;
     }
 
+    public final PhylogenyNode getNextExternalNodeWhileTakingIntoAccountCollapsedNodes() {
+        //TODO work on me ~~
+        if ( isInternal() && !isCollapse() ) {
+            throw new UnsupportedOperationException( "attempt to get next external node of an uncollapsed internal node" );
+        }
+        if ( isRoot() ) {
+            return null;
+        }
+        if ( getParent().isCollapse() ) {
+            throw new UnsupportedOperationException( "attempt to get next external node of node with a collapsed parent" );
+        }
+        // This checks if last node.
+        PhylogenyNode n = this;
+        boolean last = true;
+        while ( !n.isRoot() ) {
+            if ( !n.isLastChildNode() ) {
+                last = false;
+                break;
+            }
+            n = n.getParent();
+        }
+        if ( last ) {
+            return null;
+        }
+        int index = getChildNodeIndex();
+        PhylogenyNode previous_node = this;
+        PhylogenyNode current_node = getParent();
+        while ( !current_node.isRoot()
+                && ( current_node.isCollapse() || ( current_node.getNumberOfDescendants() == 1 ) || previous_node
+                        .isLastChildNode() ) ) {
+            index = current_node.getChildNodeIndex();
+            previous_node = current_node;
+            current_node = current_node.getParent();
+        }
+        if ( index < current_node.getNumberOfDescendants() - 1 ) {
+            current_node = current_node.getChildNode( index + 1 );
+        }
+        while ( current_node.isInternal() && !current_node.isCollapse() ) {
+            current_node = current_node.getFirstChildNode();
+        }
+        return current_node;
+    }
+
     public final NodeData getNodeData() {
         if ( _node_data == null ) {
             _node_data = new NodeData();
@@ -458,18 +487,27 @@ public class PhylogenyNode implements PhylogenyNodeI, Comparable<PhylogenyNode>
     /**
      * Returns the ID (int) of this PhylogenyNode.
      */
-    final public int getNodeId() {
+    @Override
+    final public int getId() {
         return _id;
     }
 
     /**
      * Returns the name of this node.
      */
-    final public String getNodeName() {
-        return _node_name;
+    @Override
+    final public String getName() {
+        return getNodeData().getNodeName();
+    }
+
+    final public List<PhylogenyNode> getAllDescendants() {
+        return _descendants;
     }
 
     final public int getNumberOfDescendants() {
+        if ( _descendants == null ) {
+            return 0;
+        }
         return _descendants.size();
     }
 
@@ -548,10 +586,10 @@ public class PhylogenyNode implements PhylogenyNodeI, Comparable<PhylogenyNode>
     @Override
     final public int hashCode() {
         final NodeData data = getNodeData();
-        if ( ( getNodeName().length() < 1 ) && !data.isHasSequence() && !data.isHasTaxonomy() ) {
+        if ( ( getName().length() < 1 ) && !data.isHasSequence() && !data.isHasTaxonomy() ) {
             return super.hashCode();
         }
-        int result = getNodeName().hashCode();
+        int result = getName().hashCode();
         if ( data.isHasSequence() ) {
             result ^= data.getSequence().hashCode();
         }
@@ -561,13 +599,12 @@ public class PhylogenyNode implements PhylogenyNodeI, Comparable<PhylogenyNode>
         return result;
     }
 
-    final private void init() {
-        _descendants = new ArrayList<PhylogenyNode>();
-        _parent = null;
-        _id = 0;
-        initializeData();
-    }
-
+    // final private void init() {
+    //_descendants = new ArrayList<PhylogenyNode>();
+    //  _parent = null; //TODO not needed?
+    //  _id = 0; //TODO not needed?
+    //initializeData(); //TODO not needed?
+    //}
     /**
      * Deletes data of this PhylogenyNode. Links to the other Nodes in the
      * Phylogeny, the ID and the sum of external nodes are NOT deleted. Field
@@ -575,18 +612,17 @@ public class PhylogenyNode implements PhylogenyNodeI, Comparable<PhylogenyNode>
      * 
      * @see #getLink() (Last modified: 12/20/03)
      */
-    final public void initializeData() {
-        _indicator = 0;
-        _x = 0;
-        _y = 0;
-        _node_name = "";
-        _distance_parent = PhylogenyNode.DISTANCE_DEFAULT;
-        _collapse = false;
-        _link = null;
-        _branch_data = null;
-        _node_data = null;
-    }
-
+    //    final private void initializeData() {
+    //        _indicator = 0;
+    //        _x = 0;
+    //        _y = 0;
+    //        //_node_name = "";
+    //        _distance_parent = PhylogenyDataUtil.BRANCH_LENGTH_DEFAULT;
+    //        _collapse = false;
+    //        _link = null;
+    //        _branch_data = null;
+    //        _node_data = null;
+    //    }
     /**
      * Returns whether this PhylogenyNode should be drawn as collapsed.
      */
@@ -608,6 +644,9 @@ public class PhylogenyNode implements PhylogenyNodeI, Comparable<PhylogenyNode>
      * @return true if this PhylogenyNode is external, false otherwise
      */
     final public boolean isExternal() {
+        if ( _descendants == null ) {
+            return true;
+        }
         return ( getNumberOfDescendants() < 1 );
     }
 
@@ -712,13 +751,6 @@ public class PhylogenyNode implements PhylogenyNodeI, Comparable<PhylogenyNode>
         return getNodeData().isHasEvent() && getNodeData().getEvent().isSpeciation();
     }
 
-    // ---------------------------------------------------------
-    // Iterator
-    // ---------------------------------------------------------
-    final public PhylogenyNodeIterator iterateChildNodesForward() {
-        return new ChildNodeIteratorForward( this );
-    }
-
     // ---------------------------------------------------------
     // Basic printing
     // ---------------------------------------------------------
@@ -809,6 +841,7 @@ public class PhylogenyNode implements PhylogenyNodeI, Comparable<PhylogenyNode>
      * Sets the length of the branch leading to the _parent of this
      * PhylogenyNode to double d.
      */
+    @Override
     final public void setDistanceToParent( final double d ) {
         _distance_parent = d;
     }
@@ -845,21 +878,27 @@ public class PhylogenyNode implements PhylogenyNodeI, Comparable<PhylogenyNode>
     /**
      * Sets the name of this node.
      */
+    @Override
     final public void setName( final String node_name ) {
-        _node_name = node_name;
+        getNodeData().setNodeName( node_name );
     }
 
     /**
-     * Sets the ID of this PhylogenyNode to i (int). In most cases, this number
-     * should not be set to values lower than getNodeCount().
+     * Sets the Id of this PhylogenyNode to i. In most cases, this number
+     * should not be set to values lower than getNodeCount() -- which this method
+     * does not allow.
      */
-    final protected void setNodeId( final int i ) {
+    synchronized final protected void setId( final int i ) {
+        if ( i < getNodeCount() ) {
+            throw new IllegalArgumentException( "attempt to set node id to a value less than total node count (thus violating the uniqueness of node ids)" );
+        }
         _id = i;
     }
 
     /**
      * Sets the _parent PhylogenyNode of this PhylogenyNode to n.
      */
+    @Override
     final public void setParent( final PhylogenyNode n ) {
         _parent = n;
     }
@@ -869,6 +908,9 @@ public class PhylogenyNode implements PhylogenyNodeI, Comparable<PhylogenyNode>
      * PhylogenyNode to i (int).
      */
     final public void setSumExtNodes( final int i ) {
+        if ( i < 0 ) {
+            throw new IllegalArgumentException( "attempt to set sum of external nodes to less than one" );
+        }
         _sum_ext_nodes = i;
     }
 
@@ -898,11 +940,21 @@ public class PhylogenyNode implements PhylogenyNodeI, Comparable<PhylogenyNode>
     // ---------------------------------------------------------
     // Writing of Nodes to Strings
     // ---------------------------------------------------------
-    final public String toNewHampshire( final boolean simple_nh, final boolean write_distance_to_parent ) {
+    final public String toNewHampshire( final boolean simple_nh,
+                                        final boolean write_distance_to_parent,
+                                        final NH_CONVERSION_SUPPORT_VALUE_STYLE svs ) {
         final StringBuilder sb = new StringBuilder();
         String data = "";
-        if ( !ForesterUtil.isEmpty( getNodeName() ) ) {
-            data = getNodeName();
+        if ( ( svs == NH_CONVERSION_SUPPORT_VALUE_STYLE.AS_INTERNAL_NODE_NAMES ) && !isExternal() ) {
+            if ( getBranchData().isHasConfidences()
+                    && ( getBranchData().getConfidence( 0 ).getValue() != Confidence.CONFIDENCE_DEFAULT_VALUE ) ) {
+                data = Confidence.FORMATTER.format( ForesterUtil
+                        .round( getBranchData().getConfidence( 0 ).getValue(),
+                                PhyloXmlUtil.ROUNDING_DIGITS_FOR_PHYLOXML_DOUBLE_OUTPUT ) );
+            }
+        }
+        else if ( !ForesterUtil.isEmpty( getName() ) ) {
+            data = getName();
         }
         else if ( getNodeData().isHasTaxonomy() ) {
             if ( !ForesterUtil.isEmpty( getNodeData().getTaxonomy().getTaxonomyCode() ) ) {
@@ -925,39 +977,72 @@ public class PhylogenyNode implements PhylogenyNodeI, Comparable<PhylogenyNode>
         }
         if ( data.length() > 0 ) {
             data = ForesterUtil.replaceIllegalNhCharacters( data );
-            if ( simple_nh ) {
-                if ( data.length() > 10 ) {
-                    data = data.substring( 0, 11 );
-                }
-                if ( data.indexOf( '/' ) > 0 ) {
-                    data = data.substring( 0, data.indexOf( '/' ) );
-                }
+            if ( simple_nh && ( data.length() > 10 ) ) {
+                data = data.substring( 0, 11 );
+            }
+            if ( ForesterUtil.isContainsParanthesesableNhCharacter( data ) ) {
+                sb.append( '\'' );
                 sb.append( data );
+                sb.append( '\'' );
             }
             else {
                 sb.append( data );
             }
         }
-        if ( ( getDistanceToParent() != PhylogenyNode.DISTANCE_DEFAULT ) && write_distance_to_parent ) {
+        if ( write_distance_to_parent && ( getDistanceToParent() != PhylogenyDataUtil.BRANCH_LENGTH_DEFAULT ) ) {
             sb.append( ":" );
             sb.append( getDistanceToParent() );
         }
+        if ( ( svs == NH_CONVERSION_SUPPORT_VALUE_STYLE.IN_SQUARE_BRACKETS ) && !isExternal()
+                && getBranchData().isHasConfidences()
+                && ( getBranchData().getConfidence( 0 ).getValue() != Confidence.CONFIDENCE_DEFAULT_VALUE ) ) {
+            sb.append( "[" );
+            sb.append( Confidence.FORMATTER.format( ForesterUtil
+                    .round( getBranchData().getConfidence( 0 ).getValue(),
+                            PhyloXmlUtil.ROUNDING_DIGITS_FOR_PHYLOXML_DOUBLE_OUTPUT ) ) );
+            sb.append( "]" );
+        }
         return sb.toString();
     }
 
+    /**
+     * Swaps the the two childern of a PhylogenyNode node of this Phylogeny.
+     */
+    public final void swapChildren() throws RuntimeException {
+        if ( isExternal() ) {
+            throw new RuntimeException( "attempt to swap descendants of external node" );
+        }
+        if ( getNumberOfDescendants() != 2 ) {
+            throw new RuntimeException( "attempt to swap descendants of node with " + getNumberOfDescendants()
+                    + " descendants" );
+        }
+        final PhylogenyNode a = getChildNode( 0 );
+        final PhylogenyNode b = getChildNode( 1 );
+        setChildNode( 0, b );
+        setChildNode( 1, a );
+    }
+
     /**
      * Converts this PhylogenyNode to a New Hampshire X (NHX) String
      * representation.
      */
     final public String toNewHampshireX() {
-        final StringBuffer s = new StringBuffer();
+        final StringBuffer sb = new StringBuffer();
         final StringBuffer s_nhx = new StringBuffer();
-        if ( !getNodeName().equals( "" ) ) {
-            s.append( ForesterUtil.replaceIllegalNhxCharacters( getNodeName() ) );
+        if ( !ForesterUtil.isEmpty( getName() ) ) {
+            final String name = ForesterUtil.replaceIllegalNhCharacters( getName() );
+            if ( ForesterUtil.isContainsParanthesesableNhCharacter( name ) ) {
+                sb.append( '\'' );
+                sb.append( name );
+                sb.append( '\'' );
+            }
+            else {
+                sb.append( name );
+            }
         }
-        if ( getDistanceToParent() != PhylogenyNode.DISTANCE_DEFAULT ) {
-            s.append( ":" );
-            s.append( getDistanceToParent() );
+        if ( getDistanceToParent() != PhylogenyDataUtil.BRANCH_LENGTH_DEFAULT ) {
+            sb.append( ":" );
+            sb.append( getDistanceToParent() );
         }
         if ( getNodeDataDirectly() != null ) {
             s_nhx.append( getNodeDataDirectly().toNHX() );
@@ -966,24 +1051,54 @@ public class PhylogenyNode implements PhylogenyNodeI, Comparable<PhylogenyNode>
             s_nhx.append( getBranchDataDirectly().toNHX() );
         }
         if ( s_nhx.length() > 0 ) {
-            s.append( "[&&NHX" );
-            s.append( s_nhx );
-            s.append( "]" );
+            sb.append( "[&&NHX" );
+            sb.append( s_nhx );
+            sb.append( "]" );
         }
-        return s.toString();
-    } // toNewHampshireX()
+        return sb.toString();
+    }
 
     @Override
     final public String toString() {
         final StringBuilder sb = new StringBuilder();
-        if ( !ForesterUtil.isEmpty( getNodeName() ) ) {
-            sb.append( getNodeName() );
+        if ( getNodeData().isHasTaxonomy() ) {
+            if ( !ForesterUtil.isEmpty( getNodeData().getTaxonomy().getScientificName() ) ) {
+                sb.append( getNodeData().getTaxonomy().getScientificName() );
+                sb.append( " " );
+            }
+            else if ( ( sb.length() <= 1 ) && !ForesterUtil.isEmpty( getNodeData().getTaxonomy().getTaxonomyCode() ) ) {
+                sb.append( getNodeData().getTaxonomy().getTaxonomyCode() );
+                sb.append( " " );
+            }
+            else if ( getNodeData().getTaxonomy().getIdentifier() != null ) {
+                sb.append( getNodeData().getTaxonomy().getIdentifier().toString() );
+                sb.append( " " );
+            }
+        }
+        if ( getNodeData().isHasSequence() ) {
+            if ( !ForesterUtil.isEmpty( getNodeData().getSequence().getName() ) ) {
+                sb.append( getNodeData().getSequence().getName() );
+                sb.append( " " );
+            }
+            if ( !ForesterUtil.isEmpty( getNodeData().getSequence().getSymbol() ) ) {
+                sb.append( getNodeData().getSequence().getSymbol() );
+                sb.append( " " );
+            }
+            if ( getNodeData().getSequence().getAccession() != null ) {
+                sb.append( getNodeData().getSequence().getAccession().toString() );
+                sb.append( " " );
+            }
+        }
+        if ( ( sb.length() <= 1 ) && !ForesterUtil.isEmpty( getName() ) ) {
+            sb.append( getName() );
             sb.append( " " );
         }
-        sb.append( "[" );
-        sb.append( getNodeId() );
-        sb.append( "]" );
-        return sb.toString();
+        if ( sb.length() <= 1 ) {
+            sb.append( "[" );
+            sb.append( getId() );
+            sb.append( "]" );
+        }
+        return sb.toString().trim();
     }
 
     /**
@@ -998,21 +1113,49 @@ public class PhylogenyNode implements PhylogenyNodeI, Comparable<PhylogenyNode>
      * 
      * @return total number of Nodes (int)
      */
-    final public static int getNodeCount() {
+    synchronized final public static int getNodeCount() {
         return PhylogenyNode._node_count;
     }
 
     /**
      * Increases the total number of all Nodes created so far by one.
      */
-    final private static synchronized void increaseNodeCount() {
+    synchronized final private static void increaseNodeCount() {
         ++PhylogenyNode._node_count;
     }
 
     /**
      * Sets the total number of all Nodes created so far to i (int).
      */
-    final public static synchronized void setNodeCount( final int i ) {
+    synchronized final static void setNodeCount( final int i ) {
         PhylogenyNode._node_count = i;
     }
+
+    public static PhylogenyNode createInstanceFromNhxString( final String nhx ) throws NHXFormatException,
+            PhyloXmlDataFormatException {
+        return new PhylogenyNode( nhx, PhylogenyMethods.TAXONOMY_EXTRACTION.NO, false );
+    }
+
+    public static PhylogenyNode createInstanceFromNhxString( final String nhx,
+                                                             final PhylogenyMethods.TAXONOMY_EXTRACTION taxonomy_extraction )
+            throws NHXFormatException, PhyloXmlDataFormatException {
+        return new PhylogenyNode( nhx, taxonomy_extraction, false );
+    }
+
+    public static PhylogenyNode createInstanceFromNhxString( final String nhx,
+                                                             final PhylogenyMethods.TAXONOMY_EXTRACTION taxonomy_extraction,
+                                                             final boolean replace_underscores )
+            throws NHXFormatException, PhyloXmlDataFormatException {
+        return new PhylogenyNode( nhx, taxonomy_extraction, replace_underscores );
+    }
+
+    private PhylogenyNode( final String nhx,
+                           final PhylogenyMethods.TAXONOMY_EXTRACTION taxonomy_extraction,
+                           final boolean replace_underscores ) throws NHXFormatException, PhyloXmlDataFormatException {
+        //  init();
+        NHXParser.parseNHX( nhx, this, taxonomy_extraction, replace_underscores );
+        setId( PhylogenyNode.getNodeCount() );
+        PhylogenyNode.increaseNodeCount();
+        setSumExtNodes( 1 ); // For ext node, this number is 1 (not 0!!)
+    }
 }
