@@ -1,4 +1,4 @@
-// $Id: RIO.java,v 1.26 2010/01/16 02:15:34 cmzmasek Exp $
+// $Id:
 // FORESTER -- software libraries and applications
 // for evolutionary biology research and applications.
 //
@@ -7,7 +7,7 @@
 // Copyright (C) 2000-2001 Washington University School of Medicine
 // and Howard Hughes Medical Institute
 // All rights reserved
-// 
+//
 // This library is free software; you can redistribute it and/or
 // modify it under the terms of the GNU Lesser General Public
 // License as published by the Free Software Foundation; either
@@ -17,12 +17,12 @@
 // but WITHOUT ANY WARRANTY; without even the implied warranty of
 // MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU
 // Lesser General Public License for more details.
-// 
+//
 // You should have received a copy of the GNU Lesser General Public
 // License along with this library; if not, write to the Free Software
 // Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301, USA
 //
-// Contact: cmzmasek@yahoo.com
+// Contact: phylosoft @ gmail . com
 // WWW: www.phylosoft.org/forester
 
 package org.forester.sdi;
@@ -36,6 +36,8 @@ import java.util.Arrays;
 import java.util.HashMap;
 import java.util.List;
 
+import org.forester.evoinference.matrix.distance.DistanceMatrix;
+import org.forester.io.parsers.SymmetricalDistanceMatrixParser;
 import org.forester.io.parsers.phyloxml.PhyloXmlParser;
 import org.forester.phylogeny.Phylogeny;
 import org.forester.phylogeny.PhylogenyMethods;
@@ -43,8 +45,6 @@ import org.forester.phylogeny.PhylogenyNode;
 import org.forester.phylogeny.factories.ParserBasedPhylogenyFactory;
 import org.forester.phylogeny.factories.PhylogenyFactory;
 import org.forester.phylogeny.iterators.PhylogenyNodeIterator;
-import org.forester.phylogenyinference.DistanceMatrix;
-import org.forester.phylogenyinference.SymmetricalDistanceMatrixParser;
 import org.forester.util.ForesterUtil;
 
 /*
@@ -106,7 +106,7 @@ public final class RIO {
         double distance = 0.0;
         name = name.trim();
         if ( _l == null ) {
-            throw new IllegalStateException( "Distance list has probably not been read in (successfully)." );
+            throw new RuntimeException( "Distance list has probably not been read in (successfully)." );
         }
         if ( _l.get( name ) == null ) {
             throw new IllegalArgumentException( name + " not found." );
@@ -231,9 +231,10 @@ public final class RIO {
      * @param query
      *            the sequence name of the squence whose orthologs are to be
      *            inferred
+     * @throws SDIException 
      */
     public void inferOrthologs( final File gene_trees_file, final Phylogeny species_tree, final String query )
-            throws IOException {
+            throws IOException, SDIException {
         int bs = 0;
         if ( RIO.TIME ) {
             _time = System.currentTimeMillis();
@@ -281,7 +282,8 @@ public final class RIO {
 
     // Helper method which performs the actual ortholog inference for
     // the external node with seqname query.
-    private void inferOrthologsHelper( final Phylogeny gene_tree, final Phylogeny species_tree, final String query ) {
+    private void inferOrthologsHelper( final Phylogeny gene_tree, final Phylogeny species_tree, final String query )
+            throws SDIException {
         Phylogeny assigned_tree = null;
         List<PhylogenyNode> nodes = null;
         final SDIR sdiunrooted = new SDIR();
@@ -334,7 +336,7 @@ public final class RIO {
         double o = 0.0;
         final ArrayList<String> arraylist = new ArrayList<String>();
         if ( _o_hash_maps == null ) {
-            throw new IllegalStateException( "Orthologs have not been calculated (successfully)." );
+            throw new RuntimeException( "Orthologs have not been calculated (successfully)." );
         }
         if ( threshold_orthologs < 0.0 ) {
             threshold_orthologs = 0.0;
@@ -344,7 +346,7 @@ public final class RIO {
         }
         o_hashmap = getInferredOrthologs( seq_name );
         if ( o_hashmap == null ) {
-            throw new IllegalStateException( "Orthologs for " + seq_name + " were not established." );
+            throw new RuntimeException( "Orthologs for " + seq_name + " were not established." );
         }
         if ( _seq_names.length > 0 ) {
             I: for( int i = 0; i < _seq_names.length; ++i ) {
@@ -432,13 +434,13 @@ public final class RIO {
         value1 = 0.0, value2 = 0.0, value3 = 0.0, value4 = 0.0, d = 0.0;
         final ArrayList<Tuplet> nv = new ArrayList<Tuplet>();
         if ( ( _o_hash_maps == null ) || ( _so_hash_maps == null ) || ( _sn_hash_maps == null ) ) {
-            throw new IllegalStateException( "Orthologs have not been calculated (successfully)" );
+            throw new RuntimeException( "Orthologs have not been calculated (successfully)" );
         }
         if ( ( sort < 0 ) || ( sort > 17 ) ) {
             sort = 12;
         }
         if ( ( sort > 2 ) && ( _m == null ) && ( _l == null ) ) {
-            throw new IllegalStateException( "Distance list or matrix have not been read in (successfully)" );
+            throw new RuntimeException( "Distance list or matrix have not been read in (successfully)" );
         }
         if ( threshold_orthologs < 0.0 ) {
             threshold_orthologs = 0.0;
@@ -456,7 +458,7 @@ public final class RIO {
         s_hashmap = getInferredSuperOrthologs( query_name );
         n_hashmap = getInferredSubtreeNeighbors( query_name );
         if ( ( o_hashmap == null ) || ( s_hashmap == null ) || ( n_hashmap == null ) ) {
-            throw new IllegalStateException( "Orthologs for " + query_name + " were not established" );
+            throw new RuntimeException( "Orthologs for " + query_name + " were not established" );
         }
         final StringBuffer orthologs = new StringBuffer();
         if ( _seq_names.length > 0 ) {
@@ -626,7 +628,7 @@ public final class RIO {
         String name = "", line = "";
         PrintWriter out = null;
         if ( _seq_names == null ) {
-            throw new IllegalStateException( "inferredOrthologTableToFile: seq_names_ is null." );
+            throw new RuntimeException( "inferredOrthologTableToFile: seq_names_ is null." );
         }
         Arrays.sort( _seq_names );
         out = new PrintWriter( new FileWriter( outfile ), true );
@@ -702,14 +704,14 @@ public final class RIO {
             threshold_ultra_paralogs = 100.0;
         }
         if ( _up_hash_maps == null ) {
-            throw new IllegalStateException( "Ultra paralogs have not been calculated (successfully)." );
+            throw new RuntimeException( "Ultra paralogs have not been calculated (successfully)." );
         }
         if ( return_dists && ( _m == null ) && ( _l == null ) ) {
-            throw new IllegalStateException( "Distance list or matrix have not been read in (successfully)." );
+            throw new RuntimeException( "Distance list or matrix have not been read in (successfully)." );
         }
         sp_hashmap = getInferredUltraParalogs( query_name );
         if ( sp_hashmap == null ) {
-            throw new IllegalStateException( "Ultra paralogs for " + query_name + " were not established" );
+            throw new RuntimeException( "Ultra paralogs for " + query_name + " were not established" );
         }
         if ( _seq_names.length > 0 ) {
             I: for( int i = 0; i < _seq_names.length; ++i ) {
