@@ -1,4 +1,4 @@
-// $Id: SDIR.java,v 1.5 2009/06/19 16:27:30 cmzmasek Exp $
+// $Id:
 // FORESTER -- software libraries and applications
 // for evolutionary biology research and applications.
 //
@@ -7,7 +7,7 @@
 // Copyright (C) 2000-2001 Washington University School of Medicine
 // and Howard Hughes Medical Institute
 // All rights reserved
-// 
+//
 // This library is free software; you can redistribute it and/or
 // modify it under the terms of the GNU Lesser General Public
 // License as published by the Free Software Foundation; either
@@ -17,12 +17,12 @@
 // but WITHOUT ANY WARRANTY; without even the implied warranty of
 // MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU
 // Lesser General Public License for more details.
-// 
+//
 // You should have received a copy of the GNU Lesser General Public
 // License along with this library; if not, write to the Free Software
 // Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301, USA
 //
-// Contact: cmzmasek@yahoo.com
+// Contact: phylosoft @ gmail . com
 // WWW: www.phylosoft.org/forester
 
 package org.forester.sdi;
@@ -212,6 +212,7 @@ public class SDIR {
      *            Array) must be no lower than 1
      * @return array of rooted Trees with duplication vs. speciation assigned if
      *         return_trees is set to true, null otherwise
+     * @throws SDIException 
      */
     public Phylogeny[] infer( final Phylogeny gene_tree,
                               final Phylogeny species_tree,
@@ -219,7 +220,7 @@ public class SDIR {
                               boolean minimize_sum_of_dup,
                               final boolean minimize_height,
                               final boolean return_trees,
-                              int max_trees_to_return ) {
+                              int max_trees_to_return ) throws SDIException {
         init();
         SDIse sdise = null;
         final ArrayList<Phylogeny> trees = new ArrayList<Phylogeny>();
@@ -496,17 +497,17 @@ public class SDIR {
             branches.add( new PhylogenyBranch( t.getRoot().getChildNode1(), t.getRoot().getChildNode2() ) );
             return branches;
         }
-        final Set<PhylogenyNode> one = new HashSet<PhylogenyNode>();
-        final Set<PhylogenyNode> two = new HashSet<PhylogenyNode>();
+        final Set<Integer> one = new HashSet<Integer>();
+        final Set<Integer> two = new HashSet<Integer>();
         PhylogenyNode node = t.getRoot();
-        while ( !node.isRoot() || !two.contains( node ) ) {
-            if ( !node.isExternal() && !two.contains( node ) ) {
-                if ( !one.contains( node ) && !two.contains( node ) ) {
-                    one.add( node );
+        while ( !node.isRoot() || !two.contains( node.getId() ) ) {
+            if ( !node.isExternal() && !two.contains( node.getId() ) ) {
+                if ( !one.contains( node.getId() ) && !two.contains( node.getId() ) ) {
+                    one.add( node.getId() );
                     node = node.getChildNode1();
                 }
                 else {
-                    two.add( node );
+                    two.add( node.getId() );
                     node = node.getChildNode2();
                 }
                 if ( !node.getParent().isRoot() ) {
