@@ -1,8 +1,10 @@
 package demo;
 
+
 import org.biojava.bio.structure.Structure;
 import org.biojava.bio.structure.align.util.AtomCache;
 import org.biojava.bio.structure.io.FileParsingParameters;
+import org.biojava3.structure.StructureIO;
 
 
 /** Example of how to load PDB files using the AtomCache class.
@@ -12,20 +14,41 @@ import org.biojava.bio.structure.io.FileParsingParameters;
  */
 public class DemoAtomCache {
 	public static void main(String[] args){
+		demoAtomCache();
+		demoStructureIO();
+
+	}
+
+	private static void demoStructureIO()  {
+
+
+		try {
+			Structure s1 = StructureIO.getStructure("4hhb");
+
+			Structure bioAssembly = StructureIO.getBiologicalAssembly("1stp",1);
+			
+			// do something with them...
+		} catch (Exception e){
+			e.printStackTrace();
+		}
+
+	}
+
+	private static void demoAtomCache() {
 		AtomCache cache = new AtomCache();
-		
+
 		FileParsingParameters params = cache.getFileParsingParams();
-		
+
 		params.setLoadChemCompInfo(false);
 		params.setAlignSeqRes(true);
 		params.setHeaderOnly(false);
 		params.setParseCAOnly(false);
 		params.setParseSecStruc(false);
-		
+
 		String[] pdbIDs = new String[]{"4hhb", "1cdg","5pti","1gav", "WRONGID" };
-		
+
 		for (String pdbID : pdbIDs){
-			
+
 			try {
 				Structure s = cache.getStructure(pdbID);
 				if ( s == null) {
@@ -34,13 +57,13 @@ public class DemoAtomCache {
 				}
 				// do something with the structure
 				System.out.println(s);
-				
+
 			} catch (Exception e){
 				// something crazy happened...
 				System.err.println("Can't load structure " + pdbID + " reason: " + e.getMessage());
 				e.printStackTrace();
 			}
 		}
-		
+
 	}
 }
