@@ -26,22 +26,19 @@ package org.biojava.bio.structure;
 
 import java.io.Serializable;
 import java.util.ArrayList;
-import java.util.Arrays;
 import java.util.HashMap;
 import java.util.Iterator;
 import java.util.List;
 import java.util.Map;
 
-
 import org.biojava.bio.structure.io.PDBFileReader;
 import org.biojava.bio.structure.io.SeqRes2AtomAligner;
 import org.biojava.bio.structure.io.mmcif.ChemCompGroupFactory;
 import org.biojava.bio.structure.io.mmcif.chem.PolymerType;
-import org.biojava.bio.structure.io.mmcif.chem.ResidueType;
 import org.biojava.bio.structure.io.mmcif.model.ChemComp;
+import org.biojava3.core.sequence.ProteinSequence;
 import org.biojava3.core.sequence.compound.AminoAcidCompound;
 import org.biojava3.core.sequence.template.Sequence;
-import org.biojava3.core.sequence.ProteinSequence;
 
 
 /**
@@ -64,8 +61,6 @@ public class ChainImpl implements Chain, Serializable {
 	 */
 	public static String DEFAULT_CHAIN_ID = "A";
 
-	static final List<String> waternames = Arrays.asList(new String[]{"HOH", "DOD",  "WAT"});
-
 	String swissprot_id ;
 	String name ; // like in PDBfile
 	List <Group> groups;
@@ -665,12 +660,14 @@ public class ChainImpl implements Chain, Serializable {
 			StringBuffer str = new StringBuffer();
 			for (Group g : seqResGroups) {
 				ChemComp cc = g.getChemComp();
-
-				if ( PolymerType.PROTEIN_ONLY.contains(cc.getPolymerType()) ||
+				if ( cc == null) {
+					System.err.println("Could not load ChemComp for group " + g);
+					str.append("X");
+				} else if ( PolymerType.PROTEIN_ONLY.contains(cc.getPolymerType()) ||
 						PolymerType.POLYNUCLEOTIDE_ONLY.contains(cc.getPolymerType())){
 					// an amino acid residue.. use for alignment
 					String oneLetter= ChemCompGroupFactory.getOneLetterCode(cc);
-					if ( oneLetter == null)
+					if ( oneLetter == null ||  oneLetter.length()==0  || oneLetter.equals("?"))
 						oneLetter = "X";
 					str.append(oneLetter);
 				} else {
@@ -743,38 +740,13 @@ public class ChainImpl implements Chain, Serializable {
 	 *
 	 */
 	public List<Group> getAtomLigands(){
-
-		return getLigands(getAtomGroups());
-
-	}
-
-	private List<Group> getLigands(List<Group> allGroups){
-		//String prop = System.getProperty(PDBFileReader.LOAD_CHEM_COMP_PROPERTY);
-
-		//		if ( prop == null || ( ! prop.equalsIgnoreCase("true"))){
-		//			System.err.println("You did not specify PDBFileReader.setLoadChemCompInfo, need to fetch Chemical Components anyways.");
-		//		}
-
-
-
-		List<Group> groups = new ArrayList<Group>();
-		for ( Group g: allGroups) {
-
-			ChemComp cc = g.getChemComp();
-
-			if ( ResidueType.lPeptideLinking.equals(cc.getResidueType()) ||
-					PolymerType.PROTEIN_ONLY.contains(cc.getPolymerType()) ||
-					PolymerType.POLYNUCLEOTIDE_ONLY.contains(cc.getPolymerType())
-					){
-				continue;
-			}
-			if ( ! waternames.contains(g.getPDBName())) {
-				//System.out.println("not a prot, nuc or solvent : " + g.getChemComp());
-				groups.add(g);
-			}
-		}
-
-		return groups;
+		List<Group> ligands = new ArrayList<Group>();
+		
+		for (Group g : groups)
+			if (!seqResGroups.contains(g) && !g.isWater())
+				ligands.add(g);
+		
+		return ligands;
 	}
 
 	@Override
